#include "utils.h"
#include "play.h"
#include "colors.h"
#include <util/delay.h>

void play() {
	int i,j;
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			map[i][j] = 0;
		}
	}
	
	for (i = 0; i < LAST_MOVES_COUNT; i++) {
		last_moves[i] = NO_DIRECTION;
	}
	
	//set the first two tiles
	set_random_tile();
	set_random_tile();
	draw_board();
	
	while (1) {	
		direction d = verify_direction();
		
		if (d == NO_DIRECTION) {
			drawPixel(0, 0, Color565(255,255, 255));
			continue;
		}
		
		switch (d) {
			case UP:
				move_up();
				break;
				
			case DOWN:
				move_down();
				break;
				
			case RIGHT:
				move_right();
				break;
				
			case LEFT:
				move_left();
				break;
			
		}
		draw_board();
		
		
		if (check_finished()) {
			_delay_ms(5000);
			break;
		}
		_delay_ms(800);
	
	}
	
	
}
bool check_finished() {
	
	_delay_ms(200);
	// LOSE
	if (!set_random_tile()) {
			return true;
	}
	
	int i, j;
	
	// WIN
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			if (map[i][j] == 2048) {
					return true;
			}
		}
	}
	return false;
	
}
void get_acc_direction() {
	
	double ax, ay, az, gx, gy, gz;

	mpu6050_getConvData(&ax, &ay, &az, &gx, &gy, &gz);
	
	if (ax > LIMIT) {
		last_moves[last_dir_index] = DOWN;
	} else if (ax < -LIMIT) {
		last_moves[last_dir_index] = UP;
	}

	if (ay > LIMIT) {
		last_moves[last_dir_index] = RIGHT;
	} else if (ay < -LIMIT) {
		last_moves[last_dir_index] = LEFT;
	}
	
	last_dir_index = (last_dir_index + 1) % LAST_MOVES_COUNT;
}

void move_up() {
	rotate_up();
	move_left();
	rotate_down();
}

void move_down() {
	rotate_down();
	move_left();
	rotate_up();
}

void move_right() {
	rotate_right();
	move_left();
	rotate_right();	
}

void move_left() {
	int x, y;
	for (y = 0; y < N; y++) {
		
		//allign line
		int curr_pos = 0;
		
		for (x = 0; x < N; x++) {
			if (map[y][x] != 0) {
				if (curr_pos < x) {
					map[y][curr_pos] = map[y][x];
					map[y][x] = 0;
				}
				curr_pos ++;
			}
		}
		
		if (map[y][0] == map[y][1] && map[y][2] == map[y][3]) {
			map[y][0] += map[y][1];
			map[y][1] = map[y][2] + map[y][3];
			map[y][2] = 0;
			map[y][3] = 0;
		}
		else if (map[y][0] == map[y][1]) {
			map[y][0] += map[y][1];
			map[y][1] = map[y][2];
			map[y][2] = map[y][3];
			map[y][3] = 0;
		}
		else if (map[y][1] == map[y][2]) { 
			map[y][1] += map[y][2];
			map[y][2] = map[y][3];
			map[y][3] = 0;
		}
		else if (map[y][2] == map[y][3]) {
			map[y][2] += map[y][3];
			map[y][3] = 0;
		}
	}
}

void rotate_up() {
	int aux[N][N], i, j;
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			aux[i][j] = map[i][j];
		}
	}
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			map[i][j] = aux[j][N -1 - i];
		}
	}
}

void rotate_down() {
	int aux[N][N], i, j;
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			aux[i][j] = map[i][j];
		}
	}
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			map[i][j] = aux[N - 1 - j][i];
		}
	}
}

void rotate_right() {
	int aux[N][N], i, j;
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			aux[i][j] = map[i][j];
		}
	}
	
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			map[i][j] = aux[i][N - 1 - j];
		}
	}
}

direction verify_direction() {
	int i;
	
	for (i = 1; i < LAST_MOVES_COUNT; i++) {
		if (last_moves[i] != last_moves[i - 1]) {
			return NO_DIRECTION;
		}
	}
	return last_moves[0];
	
}

tile find_empty_tile() {
	tile empty_tile = {-1, -1};
	int x, y;
	
	for (y = 0; y < N; y++) {
		for (x = 0; x < N; x++) {
			if (map[y][x] == 0) {
				empty_tile.y = y;
				empty_tile.x = x;
				return empty_tile;
			}
		}
	}
	return empty_tile;
}

bool set_random_tile() {
	
	tile empty = find_empty_tile();
	
	if(empty.x == -1) {
		// board is full
		return false;
	}
	
	
	// find a random empty tile
	do {
		empty.x = rand() % N;
		empty.y = rand() % N;
	} while (map[empty.y][empty.x] != 0);
	
	//cell value is determined with a '2' being 90% probable and a '4' 10%
	map[empty.y][empty.x] = rand() % 10 < 9 ? 2 : 4;
	draw_tile(empty.y, empty.x);
	
	return true;
}

void draw_board() {
	
	int i,j;
	for (j = 0; j < N; j++) { // y
		for (i = 0; i < N; i++) { // x
			if (map[j][i] == 0) {
				erase_tile(j, i);
			}
		}
	}
	_delay_ms(300);
	
	for (j = 0; j < N; j++) { // y
		for (i = 0; i < N; i++) { // x
			if (map[j][i] != 0) {
				draw_tile(j, i);
			}
		}
	}
}

void draw_tile(int j, int i) {
	int index = get_color_index(map[j][i]);
	int width = margins[i + 1] - margins[i] - 1;
	int height = margins[j + 1] - margins[j] - 1;
	int x = margins[i] + 1;
	int y = margins[j] + 1;
	fillRect(x, y, width, height, Color565(0, 0, 0));
	fillRect(x + 1, y + 1, width - 2, height - 2, Color565(colors[index].r, colors[index].g, colors[index].b));
	
	draw_number(map[j][i], x, y, width, height);	
}

void erase_tile(int j, int i) {
	int index = get_color_index(map[j][i]);
	int width = margins[i + 1] - margins[i] - 1;
	int height = margins[j + 1] - margins[j] - 1;
	int x = margins[i] + 1;
	int y = margins[j] + 1;
	fillRect(x, y, width, height, Color565(255, 255, 255));
}

void init_draw() {
	map[0][0] = 2;
	map[1][1] = 0;
	map[2][2] = 4;
	map[3][3] = 8;
	
	_delay_ms(500);
	draw_tile(0, 0);
	_delay_ms(200);
	draw_tile(1, 1);
	_delay_ms(200);
	draw_tile(2, 2);
	_delay_ms(200);
	draw_tile(3, 3);
	_delay_ms(500);
	fillScreen(Color565(255,255,255));
	
}